// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package email

import (
	"context"
	"fmt"
	"smart-thirdparty/errors"
	"smart-thirdparty/parser"
	pb_v1 "smart-thirdparty/pb/v1"
	"smart-thirdparty/thirdparty"

	"github.com/jaryhe/gopkgs/logger"
	jsoniter "github.com/json-iterator/go"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

// 替换encoding/json包
var json = jsoniter.ConfigCompatibleWithStandardLibrary




func SendEmail(ctx context.Context, req *pb_v1.SendEmailRequest) (reply *pb_v1.SendEmailReply, err error) {
	reply = &pb_v1.SendEmailReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	err = thirdparty.SendMail(parser.Conf.ThirdParty.MailUser,
		parser.Conf.ThirdParty.MailPass,
		parser.Conf.ThirdParty.MailHost,
		req.Email, "邮箱验证", nil, req.Content)
	if err != nil {
		logger.Error("func",
			zap.String("call", "thirdparty.SendMail"),
			zap.String("params", req.Email),
			zap.String("error", err.Error()))
		return nil, errors.EmailSendError
	}

	return reply, nil
}
