// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package notify

import (
	"context"
	"fmt"
	"smart-thirdparty/errors"
	"smart-thirdparty/parser"
	pb_v1 "smart-thirdparty/pb/v1"
	"smart-thirdparty/thirdparty"

	"github.com/jaryhe/gopkgs/logger"
	jsoniter "github.com/json-iterator/go"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

// 替换encoding/json包
var json = jsoniter.ConfigCompatibleWithStandardLibrary

// 发送短信告警
func sendSmsVerify(req *pb_v1.SendThroughVerifyRequest) error {
	var data []interface{}
	//data = append(data, parser.Conf.ThirdParty.RongLian.City)
	data = append(data, req.Company)
	if req.Status {
		data = append(data, "通过")
	} else {
		data = append(data, "不通过")
	}
	data = append(data, req.ApproveTime)
	var templateId int
	// 供应商
	if req.CompanyType == 1 {
		templateId = parser.Conf.ThirdParty.RongLian.ProviderVerifyTemplateId
		data = append(data, parser.Conf.SupplierUrl)
	} else if req.CompanyType == 2 {
		templateId = parser.Conf.ThirdParty.RongLian.CompanyVerifyTemplateId
		data = append(data, parser.Conf.EnterpriseUrl)
	} else {
		return status.Error(10003, "参数错误,企业类型不正确")
	}

	if err := thirdparty.SendTemplateSMS(req.PhoneNumber, data, templateId); err != nil {
		logger.Error("func",
			zap.String("call", "ThirdParty.SendTemplateSMS"),
			zap.String("params", req.PhoneNumber),
			zap.String("error", err.Error()))
		return errors.VerifyNoticeError
	}
	return nil
}

func SendThroughVerify(ctx context.Context, req *pb_v1.SendThroughVerifyRequest) (reply *pb_v1.SendThroughVerifyReply, err error) {
	reply = &pb_v1.SendThroughVerifyReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	if req.PhoneNumber == "" {
		return nil, status.Error(10003, "参数错误,联系人为空")
	}

	if req.Company == "" {
		return nil, status.Error(10003, "参数错误,企业名为空")
	}

	err = sendSmsVerify(req)

	return
}
