// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package vcode

import (
	"context"
	"fmt"
	"smart-thirdparty/consts"
	"smart-thirdparty/errors"
	pb_v1 "smart-thirdparty/pb/v1"
	"smart-thirdparty/utils"
	"strconv"

	"github.com/go-redis/redis"
	"github.com/jaryhe/gopkgs/cache"

	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

// CheckVcode 验证码校验接口
func CheckVcode(ctx context.Context, req *pb_v1.CheckVcodeRequest) (reply *pb_v1.CheckVcodeReply, err error) {
	reply = &pb_v1.CheckVcodeReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	// params verify
	switch {
	case len(req.PhoneNumber) != 11 || !utils.VerifyMobileFormat(req.PhoneNumber):
		return reply, status.Error(10003, "参数错误,手机号错误")
	}

	// 存入Redis
	redisKey := ""

	redisKey += consts.VCODEKEY + req.PhoneNumber

	res, err := cache.Redis().Get(redisKey)

	if err != nil && err != redis.Nil {
		logger.Error("redis",
			zap.String("call", "Get"),
			zap.String("params", redisKey),
			zap.String("error", err.Error()))
		return reply, errors.SystemError
	}

	if err == redis.Nil {
		return reply, errors.VCodeExpired
	}

	// 验证码校验失败
	i, _ := strconv.Atoi(res)
	if i != int(req.Vcode) {
		return reply, errors.VCodeVerifyFailed
	}

	// 验证以后参数该key
	_, _ = cache.Redis().Del(redisKey)

	return
}
