// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package vcode

import (
	"context"
	"fmt"
	"smart-thirdparty/consts"
	"smart-thirdparty/errors"
	"smart-thirdparty/parser"
	pb_v1 "smart-thirdparty/pb/v1"
	"smart-thirdparty/thirdparty"
	"smart-thirdparty/utils"
	"strconv"

	"github.com/jaryhe/gopkgs/cache"
	"github.com/jaryhe/gopkgs/logger"
	jsoniter "github.com/json-iterator/go"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

// 替换encoding/json包
var json = jsoniter.ConfigCompatibleWithStandardLibrary

// GetVcode 短信验证接口
func GetVcode(ctx context.Context, req *pb_v1.GetVcodeRequest) (reply *pb_v1.GetVcodeReply, err error) {
	reply = &pb_v1.GetVcodeReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	switch {
	case len(req.PhoneNumber) != 11 || !utils.VerifyMobileFormat(req.PhoneNumber):
		return reply, status.Error(10003, "参数错误,手机号错误")
	}

	// 发送验证码
	code := utils.RankVCode(1000, 9999)
	data := []interface{}{code, strconv.Itoa(consts.SMS_VALIDITY) + "分钟"}

	// request
	if err = thirdparty.SendTemplateSMS(req.PhoneNumber, data, parser.Conf.ThirdParty.RongLian.VcodeTemplateId); err != nil {
		logger.Error("func",
			zap.String("call", "ThirdParty.SendTemplateSMS"),
			zap.String("params", req.PhoneNumber),
			zap.String("error", err.Error()))
		return reply, errors.VCodeSendFailed
	}

	// 存入Redis
	redisKey := ""

	redisKey += consts.VCODEKEY + req.PhoneNumber

	// save to redis
	_, err = cache.Redis().SetEx(redisKey, consts.SMS_VALIDITY*60, code)

	if err != nil {
		logger.Error("redis",
			zap.String("call", "SetEx"),
			zap.String("params", redisKey),
			zap.String("error", err.Error()))
		return reply, errors.SystemError
	}

	reply.Vcode = uint32(code)

	return
}
